@@ -4,7 +4,7 @@ Created on May 7, 2010
 
 @author: sax
 '''
-from django.core.exceptions import ValidationError
+from django.core.exceptions import ValidationError, ObjectDoesNotExist
 from django.db import models
 from django.db.models.manager import Manager
 from django.utils.translation import gettext_lazy as _
@@ -17,14 +17,18 @@ logger = logging.getLogger("geo")
 
 
 class Currency(models.Model):
-    code = models.CharField(max_length=5)
+    code = models.CharField(max_length=5, unique=True, help_text="ISO 4217 code")
     name = models.CharField(max_length=30)
-    symbol = models.CharField(max_length=5)
+    symbol = models.CharField(max_length=5, blank=True, null=True)
 
     class Meta:
         app_label = 'geo'
         ordering = ['code', ]
 
+    def __unicode__(self):
+        return u"%s (%s)" % (self.code, self.name)
+
+
 CONTINENTS = (
     ('AF', _('Africa')),
     ('AS', _('Asia')),
@@ -38,6 +42,8 @@ CONTINENTS = (
 Regions = zip(range(1, 5), ('Africa', 'Americas', 'Asia', 'Middle East'))
 
 class CountryManager(Manager):
+    use_for_related_fields = True
+
     def _by_continent(self, c):
         return self.get_query_set().filter(continent=c)
 
@@ -75,6 +81,9 @@ class Country(models.Model):
     continent = models.CharField(choices=CONTINENTS, max_length=2)
     currency = models.ForeignKey(Currency, blank=True, null=True)
 
+#    tld = models.CharField(max_length=6, blank=True, null=True, help_text='internet tld')
+#    tz = models.IntegerField(blank=True, null=True, help_text='time zone')
+
     fullname.alphabetic_filter = True
     objects = CountryManager()
 
@@ -93,6 +102,9 @@ class Country(models.Model):
         if hasattr(item, 'country'):
             return item.country == self
 
+    def cities(self):
+        return self.location_set.cities()
+
 
 class AdministrativeAreaTypeManager(TreeManager):
     use_for_related_fields = True
@@ -127,6 +139,11 @@ class AdministrativeAreaType(MPTTModel):
             raise ValidationError(_('`%s` cannot contains same type') % self.parent)
         super(AdministrativeAreaType, self).clean()
 
+    def get_or_create(self, **kwargs):
+        return AdministrativeArea.objects.get_or_create(country=self.country, type=self, **kwargs)
+
+
+
 class AdministrativeAreaManager(TreeManager):
     use_for_related_fields = True
 
@@ -143,6 +160,7 @@ class AdministrativeArea(MPTTModel):
     """
 
     name = models.CharField(_('Name'), max_length=255, db_index=True)
+    code = models.CharField(_('Code'), max_length=5, db_index=True)
     parent = TreeForeignKey('self', null=True, blank=True, related_name='areas')
     country = models.ForeignKey(Country, related_name='areas')
     type = models.ForeignKey(AdministrativeAreaType)
@@ -151,6 +169,7 @@ class AdministrativeArea(MPTTModel):
     class Meta:
         verbose_name = _("Administrative Area")
         verbose_name_plural = _("Administrative Areas")
+        unique_together = (('name', 'country', 'type'),)
         app_label = 'geo'
         ordering = ['name']
         order_with_respect_to = 'country'
@@ -160,6 +179,7 @@ class AdministrativeArea(MPTTModel):
 
     def natural_key(self):
         return (self.country.iso_code, self.name )
+
     natural_key.dependencies = ['geo.country']
 
     def clean(self):
@@ -181,6 +201,17 @@ class AdministrativeArea(MPTTModel):
     def __contains__(self, item):
         return True
 
+class LocationManager(models.Manager):
+    use_for_related_fields = True
+
+    def get_or_none(self, *args, **kwargs):
+        try:
+            return self.get(*args, **kwargs)
+        except ObjectDoesNotExist:
+            return None
+
+    def cities(self):
+        return self.get_query_set().filter(type=Location.CITY)
 
 class Location(models.Model):
     """ Administrative location ( city, place everything with a name and Lat/Lng that
@@ -218,11 +249,13 @@ class Location(models.Model):
     acc = models.IntegerField(choices=ACCURACY, default=NONE, blank=True, null=True,
         help_text="Define the level of accuracy of lat/lng infos")
 
+    objects = LocationManager()
+
     class Meta:
         verbose_name_plural = _('Locations')
         verbose_name = _('Location')
         app_label = 'geo'
-        ordering = ("name",)
+        ordering = ('name', 'country')
         order_with_respect_to = 'country'
 
     def __unicode__(self):
@@ -230,6 +263,7 @@ class Location(models.Model):
 
     def natural_key(self):
         return self.country.natural_key() + (self.name, str(self.lat), str(self.lng))
+
     natural_key.dependencies = ['geo.country']
 
     def clean(self):
